// MBGLView.cpp : implementation file
//

#include "stdafx.h"
#include<time.h>
#include<vcg/CIMsg.h>

#include "Moebius.h"
#include "MBDoc.h"
#include "MBView.h"
#include "MBGLView.h"
#include <il/il.h>
#include <il/ilu.h>
#include <il/ilut.h>
  

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// CMBGLView

IMPLEMENT_DYNCREATE(CMBGLView, CView)

CMBGLView::CMBGLView()
{
 m_hrc=0;
 m_pDC=0;
 m_bCaptured=false;
 m_sf=1.0;
}

CMBGLView::~CMBGLView()
{
}

BEGIN_MESSAGE_MAP(CMBGLView, CView)
	ON_WM_DESTROY()
	ON_WM_SIZE()
	ON_WM_ERASEBKGND()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
  ON_WM_MOUSEWHEEL()
ON_COMMAND(ID_HELP_OPENGL, OnHelpOpengl)
END_MESSAGE_MAP()


// CMBGLView drawing

void CMBGLView::OnDraw(CDC* pDC)
{	
	CMBDoc * pd = GetDocument();
	SetGL();
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glInit();
	glMatrixMode(GL_PROJECTION);	
	glLoadIdentity();
	gluPerspective(60,aspect,.4,25);

	// Inizializzazione della Modelview Matrix
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();	
  
	// World To Camera Transformation
	gluLookAt(0,0,10,0,0,0,0,1,0);
	glScalef(m_sf,m_sf,m_sf);
		
	// Apply the trackball
	glMultMatrix(m_matRot);
  pd->Scene.root.glDraw(pd->ElapsedSec);

	// Calcolo e stampa fps
	fps.Add();
	CString buf;
	buf.Format("Fps %4.2f",fps.LastFPS());
	cf.OrthoPrint(buf, Point3f(10,10,0));
	
		
	SwapBuffers(m_pDC->GetSafeHdc());
}


// CMBGLView diagnostics

#ifdef _DEBUG
void CMBGLView::AssertValid() const
{
	CView::AssertValid();
}

void CMBGLView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}
// CMBView diagnostics

CMBDoc* CMBGLView::GetDocument() const // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CMBDoc)));
	return (CMBDoc*)m_pDocument;
}
#endif //_DEBUG


// CMBGLView message handlers
void CMBGLView::glInit()
{
	// Abilita il la possibilita' di cambiare il alcune componenti colore del 
	// materiale tramite un semplice glcolor, anzich fare glMaterial(...)
	glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);

	// Abilita il calcolo dell'illuminazione
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	// anche le normali attraversano la pipeline, quindi devono essere  
	// rinormalizzate, prima del calcolo dell'illuminazione, altrimenti 
	// il calcolo dell'illuminazione viene sbagliato.
	glEnable(GL_NORMALIZE); 
	
	// Abilita il test sullo zbuffer e quindi l'eliminazione delle 
	// superfici nascoste
	glEnable(GL_DEPTH_TEST); 	

	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK);
	
}


BOOL CMBGLView::SetupPixelFormat( CDC * pDC )
{
    static PIXELFORMATDESCRIPTOR pfd = 
	 {
		sizeof(PIXELFORMATDESCRIPTOR),  //  size of this pfd
			1,                     // version number
			PFD_DRAW_TO_WINDOW |   // support window
			PFD_SUPPORT_OPENGL |   // support OpenGL
			PFD_DOUBLEBUFFER,      // double buffered
			PFD_TYPE_RGBA,         // RGBA type 
			24,                    // 24-bit color depth 
			0, 0, 0, 0, 0, 0,      // color bits ignored
			0,                     // no alpha buffer
			0,                     // shift bit ignored
			0,                     // no accumulation buffer
			0, 0, 0, 0,            // accum bits ignored
			32,                    // 32-bit z-buffer
			0,                     // no stencil buffer
			0,                     // no auxiliary buffer
			PFD_MAIN_PLANE,        // main layer
			0,                     // reserved
			0, 0, 0                // layer masks ignored
	 };
    int pixelformat;

    if ( (pixelformat = ChoosePixelFormat(pDC->GetSafeHdc(), &pfd)) == 0 )
    {
        AfxMessageBox("ChoosePixelFormat failed");
	    return FALSE;
    }

    if (SetPixelFormat(pDC->GetSafeHdc(), pixelformat, &pfd) == FALSE)
    {
        AfxMessageBox("SetPixelFormat failed");
	    return FALSE;
    }

    return TRUE;
}


BOOL CMBGLView::Create(LPCTSTR lpszClassName, LPCTSTR lpszWindowName, DWORD dwStyle, const RECT& rect, CWnd* pParentWnd, UINT nID, CCreateContext* pContext)
{
  if( ! CView::Create(lpszClassName, lpszWindowName, dwStyle, 
		rect, pParentWnd, nID, pContext)) 	return false;

  m_pDC = new CClientDC(this);
  ASSERT(m_pDC != NULL);

  // Setta il pixel format
  if (!SetupPixelFormat(m_pDC)) return FALSE;
  // Crea e setta il contesto OPENGL
  m_hrc = wglCreateContext(m_pDC->GetSafeHdc());
  if(m_hrc==NULL){
		AfxMessageBox("OpenGL contest fail");
		return FALSE;
	}
  return TRUE;
}

void CMBGLView::OnDestroy()
{
  CView::OnDestroy();
  SetGL();
  glFinish();
  wglMakeCurrent(NULL,  NULL);
  if (m_hrc) ::wglDeleteContext(m_hrc);
  if (m_pDC) delete m_pDC;
}

BOOL CMBGLView::OnEraseBkgnd(CDC* pDC)
{
  return TRUE;
}

void CMBGLView::OnSize(UINT nType, int cx, int cy)
{
	if(SetGL()){
	glMatrixMode (GL_PROJECTION);   
	glLoadIdentity ();
	aspect=(float)cx/(float)cy;
	glViewport (0, 0, (GLsizei) cx, (GLsizei) cy); 
	m_tb.Resize( cx,cy );
	}
}

void CMBGLView::OnInitialUpdate()
{
	CView::OnInitialUpdate();
  CMBDoc * pd = GetDocument();
	SetGL();
	extgl_Initialize();
	ilInit();
	iluInit();
	ilutRenderer(ILUT_OPENGL);
	tiRing=ilutGLLoadImage("data/spheremap2.jpg");
	ilutGLBuildMipmaps();
	//tiBall=ilutGLLoadImage("ball.png");
	//ilutGLBuildMipmaps();
	
	ILuint ili;
	ilGenImages(1,&ili);
	ilBindImage(ili);
	ilLoad(IL_TYPE_UNKNOWN ,"data/ball.png");
	glGenTextures(1,&tiBall);
	glBindTexture(GL_TEXTURE_2D,tiBall);
	glTexImage2D(GL_TEXTURE_2D,0,3,64,64,0,GL_RGB,GL_UNSIGNED_BYTE,ilGetData());
	gluBuild2DMipmaps(GL_TEXTURE_2D,3,64,64,GL_RGB,GL_UNSIGNED_BYTE,ilGetData());

	cm.LoadOld("data/uffizi.jpg");
	CRect rc;
	GetClientRect(&rc);
	m_tb.Init(rc.right,rc.bottom);
	m_matRot.SetIdentity();
	cf.Init("rial",20);
}

void CMBGLView::OnLButtonDown(UINT nFlags, CPoint point)
{
	if(nFlags  & MK_CONTROL   )
	{
		CMBDoc *pd=GetDocument();
		SetGL();
		int ress,resb;
		TRACE("Picking\n");
		int pos=pd->Scene.Pick(point.x,point.y,pd->ElapsedSec);
		if(pos>0) {
			pd->m->Ind2Sec(pos,ress,resb);
			pd->m->Sec(ress,resb).c=Point3f(.3f,1,.3f);
		}
	}
	if(!m_bCaptured)
	{
		m_tb.MouseDown( point.x, point.y, 0 );
		SetCapture();
		m_bCaptured = TRUE;
	}

	CView::OnLButtonDown(nFlags, point);
}

void CMBGLView::OnLButtonUp(UINT nFlags, CPoint point)
{
	if(m_bCaptured)
	{
		m_tb.MouseUp(point.x,point.y);
		ReleaseCapture();
		m_bCaptured = FALSE;
	}

	CView::OnLButtonUp(nFlags, point);
}

void CMBGLView::OnMouseMove(UINT nFlags, CPoint point)
{
	if(m_bCaptured)
	{
		m_tb.CalcRotMatrix( m_matRot, point.x, point.y );
		GetDocument()->UpdateTime();
		Invalidate();
	}	
	CView::OnMouseMove(nFlags, point);
}


void CMBGLView::OnHelpOpengl()
{
	const char *msg=(char *)glGetString(GL_EXTENSIONS);
	AfxMessageBox(msg);
}


BOOL CMBGLView::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt)
{
	if(zDelta>0) m_sf*=1.2f;
	if(zDelta<0) m_sf/=1.2f;
	Invalidate();
	return CView::OnMouseWheel(nFlags, zDelta, pt);
}
