#pragma once
#include<code/extgl.h>

#include <vector>
#include <vcg/Point3.h>
#include <vcg/Matrix44.h>
#include <vcg/CIMesh.h>
#include <vcg/SG.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

class MoebiusStrip :  public CSGNode
{
public:
	class Block
	{
	 public:
	 };
 
 class Pos
 {
 public:
	 float ThetaDeg;
	 int CurSide;
 };
 
 // sposta la posizione di dist lungo la sup dell'anello
 void Move(Pos &p, float DistDeg);

 // genera la trasformazione opengl che porta il sistema di riferimento 
 // con l'origine sulla superfice del nastro, z up y front e x destra;
 void Transform(const Pos &p);
  
	MoebiusStrip(void);
	~MoebiusStrip(void);
  virtual void XMLWrite(FILE *fp);
	virtual void XMLRead(Xml &xml, CSG *Base);


	void Ind2Sec(const SecInd, int &SideInd, int &BlockInd)
	{
		SideInd=SecInd/BlockNum;
		BlockInd=SecInd%BlockNum;
		assert(SideInd>=0 && SideInd<SideNum);
		assert(BlockInd>=0 && BlockInd<BlockNum);
	}

	int Sec2Ind(int SideInd, int BlockInd)const 
	{
		return SideInd*BlockNum+BlockInd;
	}
	CIMesh &Sec(int s, int b)
	{
		assert(s>=0 && s< SideNum);
		assert(b>=0 && b< BlockNum);
		return Ring[Sec2Ind(s,b)];
	}

	void Generate();
	vector<CIMesh> Ring;
	
	virtual void glDraw(const float DocTime);
	//CIMesh Ring;
	// parameters

	int SideNum;  // 2 = strip, 3 triangular section ecc.
	float Turns;  // espresso in 2pi / SideNum. 
  float Radius; // del centro della strip
	float InnerRadius;  // largezza lato strip;
	float FilletRatio;  // lato "verticale"
  int BlockNum;  // in quanti pezzetti  diviso un anello.
	float StartTwistDeg;
	int StepNum;  // con quanti step si approssima un blocco
	
	bool Empty() const {
		if(Ring.empty()) return true;
		return Ring[0].Empty();
	}
	void Clear();
	void GenerateRing(const vector<Point3f> &SectionP, const vector<Point3f> &SectionN);
	void GenerateBlock(const vector<Point3f> &SP,  const vector<Point3f> &SN,int side, int step, CIMesh &m);
  void GenerateSideSection( vector<Point3f> &SP,  vector<Point3f> &SN);
	int PickElement(int x, int y, int& SideInd, int& BlockInd);

};


class CSGMb : public CSG
{
public:
	CSGMb() {m=0;}
	virtual CSGNode *Allocate(const string &classname);
  virtual void Clear();
	MoebiusStrip *m;
};


class CSGAnimMoebiusTransf  : public CSGNode
{
public:
	CSGMb *b;
	virtual	~CSGAnimMoebiusTransf(void){};
	float AngularSpeedDPS; //Degree Per Sec;
  float StartAngleDeg; 
	int StartSide;

	virtual void glDraw(const float DocTime);
	virtual void XMLWrite(FILE *fp);
	virtual void XMLRead(Xml &xml, CSG *Base);
};



class CSGSphereMesh : public CSGNode, CIMesh
{
public:
	float Radius;
	int Parallel;
	int Meridian;
	CSGSphereMesh(){};
	CSGSphereMesh(float r,int p,int m){
		Radius=r;Parallel=p;Meridian=m;
		CIShape::Sphere(*this,r,p,m);};
	virtual	~CSGSphereMesh(void){};
	
	virtual void glDraw(const float DocTime);
	virtual void XMLWrite(FILE *fp);
	virtual void XMLRead(Xml &xml, CSG *Base);
};
